import fitz  # PyMuPDF
from PIL import Image
import os
from docx import Document
import sys

config = {
    'threshold': 100,
    'blank_height': 10,
    'min_height': 37,
    'bottom_margin': 0,
    'template_docx': './_template.docx',
    'dpi': 300
}

def is_blank(row, threshold=config['threshold']):
    return all(sum(pixel) / 3 >= threshold for pixel in row)

def find_segments(pixels, width, height):
    segments = []
    start = None
    blanks = 0
    blank_height = config['blank_height']

    for y in range(height):
        if is_blank([pixels[y * width + x] for x in range(width)]):
            blanks += 1
            if start is not None and blanks > blank_height:
                segments.append({'start': start, 'end': y})
                start = None
        else:
            if start is None:
                start = y
            blanks = 0

    if start is not None:
        segments.append({'start': start, 'end': height})

    return segments

def save_segments(image_path, output_dir, counter):
    image = Image.open(image_path)
    width, height = image.size
    pixels = list(image.getdata())
    bottom_margin = config['bottom_margin']
    
    cropped_image = image.crop((0, 0, width, max(0, height - bottom_margin)))
    non_blank_segments = find_segments(pixels, width, cropped_image.height)

    segment_paths = []
    for idx, seg in enumerate(non_blank_segments, start=1):
        segment_height = seg['end'] - seg['start']
        if segment_height >= config['min_height']:
            segment_img = Image.new('RGB', (width, segment_height))
            segment_img.putdata(pixels[seg['start']*width:seg['end']*width])
            segment_output_path = f"{output_dir}/segment_{counter}.png"
            segment_img.save(segment_output_path)
            print(f"Saved {segment_output_path}")
            segment_paths.append(segment_output_path)
            counter += 1

    return segment_paths, counter

def pdf_to_imgs(pdf_path, output_dir):
    pdf = fitz.open(pdf_path)
    return [f"{output_dir}/page_{i+1}.jpg" for i, page in enumerate(pdf) 
            for pix in [page.get_pixmap(dpi=config['dpi'])] 
            for _ in [pix.save(f"{output_dir}/page_{i+1}.jpg")]]

def process_pdf(pdf_path):
    base_name = os.path.splitext(os.path.basename(pdf_path))[0]
    output_dir = f"./_temp/{base_name}_segments"
    os.makedirs(output_dir, exist_ok=True)
    
    doc = Document(config['template_docx'])
    segment_counter = 1
    
    for img_path in pdf_to_imgs(pdf_path, output_dir):
        print(f"Processing {img_path}...")
        segments, segment_counter = save_segments(img_path, output_dir, segment_counter)
        for segment in segments:
            doc.add_picture(segment)        
        os.remove(img_path)
        print(f"Deleted {img_path}")

    doc.save(f"./__output_{base_name}.docx")
    print(f"Saved final document.")

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python .py <path_to_pdf>")
        sys.exit(1)
    
    process_pdf(sys.argv[1])